"use strict";
//! Credit goes to https://stackoverflow.com/a/44123368/4554883
Object.defineProperty(exports, "__esModule", { value: true });
exports.otag = void 0;
const defer_1 = require("./defer");
/** @private */
class Carrier {
    constructor() {
        this.deferred = (0, defer_1.default)();
        this.finished = false;
        // has to be a function expression
        this.doComplete = () => {
            this.finished = true;
            this.deferred.resolve();
        };
    }
    // eslint-disable-next-line @typescript-eslint/promise-function-async
    getValue() {
        return this.deferred;
    }
    isFinished() {
        return this.finished;
    }
    // eslint-disable-next-line @typescript-eslint/promise-function-async
    spawnDeferred() {
        const deferred = this.deferred;
        this.deferred = (0, defer_1.default)();
        return deferred;
    }
    next(value) {
        setImmediate(() => {
            this.spawnDeferred().resolve(value);
        });
    }
    convertToError(value) {
        if (value instanceof Error) {
            return value;
        }
        return new Error(String(value));
    }
    error(value) {
        const error = this.convertToError(value);
        setImmediate(() => {
            this.spawnDeferred().reject(error);
        });
    }
    complete() {
        setImmediate(this.doComplete);
    }
}
async function* otag(observable) {
    const valueCarrier = new Carrier();
    const subscription = observable.subscribe(valueCarrier);
    try {
        while (true) {
            const value = await valueCarrier.getValue();
            if (valueCarrier.isFinished()) {
                break;
            }
            yield value;
        }
    }
    finally {
        subscription.unsubscribe();
    }
}
exports.otag = otag;
//# sourceMappingURL=otag.js.map